import { Request, Response } from "express";
import BoatType from "../db/models/BoatType";
import Boat from "../db/models/Boat";

//createBoatTypeController
const createBoatTypeController = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role == "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }

    const newBoatTypeInput = req.body;

    const newBoatType = await BoatType.create(newBoatTypeInput);

    if (newBoatType) {
      return res.status(201).json({
        success: true,
        result: {
          id: newBoatType.id,
          name: newBoatType.name,
          seats: newBoatType.seats,
        },
      });
    }
  } catch (error) {
    console.error(error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//show all BoatTypes
const showAllBoatTypes = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const allBoatTypes = await BoatType.findAll();
    return res.status(200).send({
      success: true,
      result: allBoatTypes.map((boattype) => {
        return { id: boattype.id, name: boattype.name, seats: boattype.seats };
      }),
    });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//delete specific BoatType using given id
const deleteBoatTypeById = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const givenId = req.params.id;
    const boatToDelete = await BoatType.destroy({
      where: {
        id: givenId,
      },
    });
    if (boatToDelete == 0) {
      return res
        .status(404)
        .json({ success: false, error: "BoatIdDoesNotExist" });
    }
    return res.status(204).json({ success: true });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//show boatType using boat's id
const showBoatTypeById = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const givenId = req.params.id;
    const boat = await Boat.findByPk(givenId);
    if (boat) {
      const boattypeid = boat.boattype;
      const boatname = await BoatType.findByPk(boattypeid);
      return res
        .status(200)
        .json({ success: true, result: { name: boatname.name } });
    }
    return res.status(404).json({ success: false, error: "boatIdNotFound" });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//update boatType by given id
const updateBoatTypeById = async (req: Request, res: Response) => {
  try {
    //check authority
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }

    //get new input
    const input = req.body;

    //return 200 with empty response if no data was given
    if (Object.keys(input).length === 0) {
      return res
        .status(200)
        .json({ success: true, result: {}, message: "noInputFound" });
    }

    //get id
    const givenId = req.params.id;

    //check if boatType can be found using givenId
    const foundBoatType = await BoatType.findByPk(givenId);

    if (!foundBoatType) {
      return res
        .status(404)
        .json({ success: false, error: "boatTypeIdNotFound" });
    }

    //try to update
    const updatedBoatType = await BoatType.update(input, {
      where: {
        id: givenId,
      },
      returning: true,
    });

    //return after updating
    const boatTypeDataAfterUpdate = updatedBoatType[1][0];
    return res.status(200).json({
      success: true,
      result: {
        id: boatTypeDataAfterUpdate.id,
        name: boatTypeDataAfterUpdate.name,
        seats: boatTypeDataAfterUpdate.seats,
      },
    });
  } catch (error) {
    console.error(error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

const boatTypeControllers = {
  showAllBoatTypes,
  createBoatTypeController,
  deleteBoatTypeById,
  showBoatTypeById,
  updateBoatTypeById,
};

export default boatTypeControllers;
